@@ -24,7 +24,7 @@
  */
 
 #include "archive_platform.h"
-__FBSDID("$FreeBSD: src/lib/libarchive/archive_read_support_compression_all.c,v 1.6 2007/01/09 08:05:55 kientzle Exp $");
+__FBSDID("$FreeBSD: src/lib/libarchive/archive_read_support_compression_all.c,v 1.7 2008/12/06 06:45:15 kientzle Exp $");
 
 #include "archive.h"
 
@@ -38,6 +38,12 @@ archive_read_support_compression_all(struct archive *a)
 	archive_read_support_compression_compress(a);
 #if HAVE_ZLIB_H
 	archive_read_support_compression_gzip(a);
+#endif
+#if HAVE_LZMADEC_H
+	/* LZMA bidding is subject to false positives because
+	 * the LZMA file format has a very weak signature.  It
+	 * may not be feasible to include LZMA detection here. */
+	/* archive_read_support_compression_lzma(a); */
 #endif
 	return (ARCHIVE_OK);
 }
