@@ -1,15 +1,21 @@
 DEFINE_TEST(test_acl_basic)
+DEFINE_TEST(test_acl_freebsd)
 DEFINE_TEST(test_acl_pax)
 DEFINE_TEST(test_archive_api_feature)
 DEFINE_TEST(test_bad_fd)
+DEFINE_TEST(test_compat_bzip2)
 DEFINE_TEST(test_compat_gtar)
+DEFINE_TEST(test_compat_gzip)
 DEFINE_TEST(test_compat_tar_hardlink)
 DEFINE_TEST(test_compat_zip)
 DEFINE_TEST(test_empty_write)
 DEFINE_TEST(test_entry)
 DEFINE_TEST(test_entry_strmode)
+DEFINE_TEST(test_fuzz)
 DEFINE_TEST(test_link_resolver)
-DEFINE_TEST(test_pax_filename_encoding)
+DEFINE_TEST(test_pax_filename_encoding_1)
+DEFINE_TEST(test_pax_filename_encoding_2)
+DEFINE_TEST(test_pax_filename_encoding_3)
 DEFINE_TEST(test_read_compress_program)
 DEFINE_TEST(test_read_data_large)
 DEFINE_TEST(test_read_extract)
@@ -23,12 +29,14 @@ DEFINE_TEST(test_read_format_cpio_svr4_gzip)
 DEFINE_TEST(test_read_format_cpio_svr4c_Z)
 DEFINE_TEST(test_read_format_empty)
 DEFINE_TEST(test_read_format_gtar_gz)
+DEFINE_TEST(test_read_format_gtar_lzma)
 DEFINE_TEST(test_read_format_gtar_sparse)
 DEFINE_TEST(test_read_format_iso_gz)
 DEFINE_TEST(test_read_format_isorr_bz2)
 DEFINE_TEST(test_read_format_mtree)
 DEFINE_TEST(test_read_format_pax_bz2)
 DEFINE_TEST(test_read_format_tar)
+DEFINE_TEST(test_read_format_tar_empty_filename)
 DEFINE_TEST(test_read_format_tbz)
 DEFINE_TEST(test_read_format_tgz)
 DEFINE_TEST(test_read_format_tz)
@@ -39,17 +47,22 @@ DEFINE_TEST(test_read_position)
 DEFINE_TEST(test_read_truncated)
 DEFINE_TEST(test_tar_filenames)
 DEFINE_TEST(test_tar_large)
+DEFINE_TEST(test_ustar_filenames)
+DEFINE_TEST(test_write_compress)
 DEFINE_TEST(test_write_compress_program)
 DEFINE_TEST(test_write_disk)
 DEFINE_TEST(test_write_disk_hardlink)
 DEFINE_TEST(test_write_disk_perms)
 DEFINE_TEST(test_write_disk_secure)
+DEFINE_TEST(test_write_disk_times)
 DEFINE_TEST(test_write_format_ar)
 DEFINE_TEST(test_write_format_cpio)
 DEFINE_TEST(test_write_format_cpio_empty)
 DEFINE_TEST(test_write_format_cpio_newc)
 DEFINE_TEST(test_write_format_cpio_odc)
+DEFINE_TEST(test_write_format_pax)
 DEFINE_TEST(test_write_format_shar_empty)
 DEFINE_TEST(test_write_format_tar)
 DEFINE_TEST(test_write_format_tar_empty)
+DEFINE_TEST(test_write_format_tar_ustar)
 DEFINE_TEST(test_write_open_memory)
