@@ -23,7 +23,7 @@
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 #include "test.h"
-__FBSDID("$FreeBSD: src/lib/libarchive/test/test_read_format_gtar_sparse.c,v 1.7 2008/01/01 22:28:04 kientzle Exp $");
+__FBSDID("$FreeBSD: src/lib/libarchive/test/test_read_format_gtar_sparse.c,v 1.11 2008/12/06 05:58:24 kientzle Exp $");
 
 
 struct contents {
@@ -181,36 +181,39 @@ struct archive_contents {
 static void
 verify_archive_file(const char *name, struct archive_contents *ac)
 {
-	char path[512];
 	struct archive_entry *ae;
 	int err;
 	/* data, size, offset of next expected block. */
 	struct contents expect;
 	/* data, size, offset of block read from archive. */
 	struct contents actual;
+	const void *p;
 	struct archive *a;
 
-	sprintf(path, "%s/%s", refdir, name);
+	extract_reference_file(name);
 
 	assert((a = archive_read_new()) != NULL);
 	assert(0 == archive_read_support_compression_all(a));
 	assert(0 == archive_read_support_format_tar(a));
-	failure("Can't open %s", path);
-	assert(0 == archive_read_open_filename(a, path, 3));
+	failure("Can't open %s", name);
+	assert(0 == archive_read_open_filename(a, name, 3));
 
 	while (ac->filename != NULL) {
 		struct contents *cts = ac->contents;
 
-		assertEqualIntA(a, 0, archive_read_next_header(a, &ae));
+		if (!assertEqualIntA(a, 0, archive_read_next_header(a, &ae))) {
+			assert(0 == archive_read_finish(a));
+			return;
+		}
 		failure("Name mismatch in archive %s", name);
 		assertEqualString(ac->filename, archive_entry_pathname(ae));
 
 		expect = *cts++;
 		while (0 == (err = archive_read_data_block(a,
-				 (const void **)&actual.d,
-				 &actual.s, &actual.o))) {
+				 &p, &actual.s, &actual.o))) {
+			actual.d = p;
 			while (actual.s > 0) {
-				char c = *(const char *)actual.d;
+				char c = *actual.d;
 				if(actual.o < expect.o) {
 					/*
 					 * Any byte before the expected
@@ -249,7 +252,7 @@ verify_archive_file(const char *name, struct archive_contents *ac)
 		assertEqualIntA(a, err, ARCHIVE_EOF);
 		failure("%s: Size returned at EOF must be zero", name);
 		assertEqualInt(actual.s, 0);
-#if ARCHIVE_VERSION_STAMP < 1009000
+#if ARCHIVE_VERSION_NUMBER < 1009000
 		/* libarchive < 1.9 doesn't get this right */
 		skipping("offset of final sparse chunk");
 #else
@@ -264,10 +267,10 @@ verify_archive_file(const char *name, struct archive_contents *ac)
 	assertEqualIntA(a, ARCHIVE_EOF, err);
 
 	assert(0 == archive_read_close(a));
-#if ARCHIVE_API_VERSION > 1
-	assert(0 == archive_read_finish(a));
-#else
+#if ARCHIVE_VERSION_NUMBER < 2000000
 	archive_read_finish(a);
+#else
+	assert(0 == archive_read_finish(a));
 #endif
 }
 
@@ -282,7 +285,7 @@ DEFINE_TEST(test_read_format_gtar_sparse)
 	 * libarchive < 1.9 doesn't support the newer --posix sparse formats
 	 * from GNU tar 1.15 and later.
 	 */
-#if ARCHIVE_VERSION_STAMP < 1009000
+#if ARCHIVE_VERSION_NUMBER < 1009000
 	skipping("read support for GNUtar --posix sparse formats");
 #else
 	/*
